/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-bandwidth-group-list.h - Holds a list of bandwidth groups.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_BANDWIDTH_GROUP_LIST_H
#define TNT_BANDWIDTH_GROUP_LIST_H

#define TNT_BANDWIDTH_GROUP_LIST(object) ((TntBandwidthGroupList *) object)

struct _TntBandwidthGroupList
{
  FluxObject         object;

  GSList            *group_add_funcs;
  GSList            *group_update_funcs;
  GSList            *group_remove_funcs;
  GSList            *group_action_funcs;

  GSList            *groups;

  gint               iterate_id;
};

void                   tnt_bandwidth_group_list_init                  (TntBandwidthGroupList *bwgl);
TntBandwidthGroupList *tnt_bandwidth_group_list_new                   (void);
void                   tnt_bandwidth_group_list_finalize              (FluxObject *bwgl);

void                   tnt_bandwidth_group_list_add_group_add_func    (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthGroupListFunc *func,
                                                                       gpointer data);
void                   tnt_bandwidth_group_list_add_group_update_func (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthGroupListFunc *func,
                                                                       gpointer data);
void                   tnt_bandwidth_group_list_add_group_remove_func (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthGroupListFunc *func,
                                                                       gpointer data);
void                   tnt_bandwidth_group_list_add_group_action_func (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthActionFunc *func,
                                                                       gpointer data);

void                   tnt_bandwidth_group_list_remove_group_func     (TntBandwidthGroupList *bwgl,
                                                                       gpointer func,
                                                                       gpointer data);

void                   tnt_bandwidth_group_list_updated               (TntBandwidthGroupList *bwgl,
                                                                       gint group_n);

gint                   tnt_bandwidth_group_list_get_n_groups          (TntBandwidthGroupList *bwgl);

TntBandwidthGroup     *tnt_bandwidth_group_list_get_group_n           (TntBandwidthGroupList *bwgl,
                                                                       gint group_n);
gint                   tnt_bandwidth_group_list_add_group             (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthGroup *group);
void                   tnt_bandwidth_group_list_remove_group          (TntBandwidthGroupList *bwgl,
                                                                       TntBandwidthGroup *group);

TntBandwidthGroup     *tnt_bandwidth_group_list_find_group_by_edge    (TntBandwidthGroupList *bwgl,
                                                                       TntEdge *edge);
gboolean               tnt_bandwidth_group_list_add_edge              (TntBandwidthGroupList *bwgl,
                                                                       TntEdge *edge);
gboolean               tnt_bandwidth_group_list_remove_edge           (TntBandwidthGroupList *bwgl,
                                                                       TntEdge *edge);

TntBandwidthGroup     *tnt_bandwidth_group_list_find_group_by_name    (TntBandwidthGroupList *bwgl,
                                                                       const gchar *name);
TntBandwidthGroup     *tnt_bandwidth_group_list_find_group_by_ip      (TntBandwidthGroupList *bwgl,
                                                                       TntNodeIP4 ip);

#endif  /* TNT_BANDWIDTH_GROUP_LIST_H */
