/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-bandwidth-group.h - Describes a bandwidth goal and who belongs to it.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_BANDWIDTH_GROUP_H
#define TNT_BANDWIDTH_GROUP_H

#define TNT_BANDWIDTH_GROUP(object) ((TntBandwidthGroup *) object)

typedef struct
{
  guint cps [TNT_STREAM_TYPE_NUM] [TNT_TRANSFER_DIRECTION_NUM];
}
TntBandwidth;

struct _TntBandwidthGroup
{
  FluxObject              object;

  gchar                  *name;
  TntNodeIP4              ip, netmask;
  gint                    goal_cps [TNT_TRANSFER_DIRECTION_NUM];

  GList                  *edge_list;

  TntBandwidthAdjustment  bandwidth_adjust_type         [TNT_TRANSFER_DIRECTION_NUM];
  TntTimeStamp            bandwidth_adjust_changed_last [TNT_TRANSFER_DIRECTION_NUM];

  TntBandwidthActionFunc *action_func;
  gpointer                action_func_data;

  TntBandwidthGroupFunc  *update_func;
  gpointer                update_func_data;
};

void               tnt_bandwidth_group_init             (TntBandwidthGroup *bandwidth_group);
TntBandwidthGroup *tnt_bandwidth_group_new              (void);
void               tnt_bandwidth_group_finalize         (FluxObject *bandwidth_group);

void               tnt_bandwidth_group_iterate          (TntBandwidthGroup *bandwidth_group);

const gchar       *tnt_bandwidth_group_get_name         (TntBandwidthGroup *bandwidth_group);
void               tnt_bandwidth_group_set_name         (TntBandwidthGroup *bandwidth_group,
                                                         const gchar *name);

void               tnt_bandwidth_group_get_accept       (TntBandwidthGroup *bandwidth_group,
                                                         gchar **ip, gchar **netmask);
void               tnt_bandwidth_group_set_accept       (TntBandwidthGroup *bandwidth_group,
                                                         const gchar *ip, const gchar *netmask);

gint               tnt_bandwidth_group_get_goal         (TntBandwidthGroup *bandwidth_group,
                                                         TntTransferDirection dir);
void               tnt_bandwidth_group_set_goal         (TntBandwidthGroup *bandwidth_group,
                                                         TntTransferDirection dir, gint cps);

void               tnt_bandwidth_group_set_action_func  (TntBandwidthGroup *bandwidth_group,
                                                         TntBandwidthActionFunc *func,
                                                         gpointer data);
void               tnt_bandwidth_group_set_update_func  (TntBandwidthGroup *bandwidth_group,
                                                         TntBandwidthGroupFunc *func,
                                                         gpointer data);

gint               tnt_bandwidth_group_get_cps          (TntBandwidthGroup *bandwidth_group,
                                                         TntTransferDirection dir);

gboolean           tnt_bandwidth_group_have_edge        (TntBandwidthGroup *bandwidth_group, TntEdge *edge);
gboolean           tnt_bandwidth_group_add_edge         (TntBandwidthGroup *bandwidth_group, TntEdge *edge);
gboolean           tnt_bandwidth_group_remove_edge      (TntBandwidthGroup *bandwidth_group, TntEdge *edge);

gboolean           tnt_bandwidth_group_match_ip         (TntBandwidthGroup *bandwidth_group, TntNodeIP4 ip);
gboolean           tnt_bandwidth_group_match_edge       (TntBandwidthGroup *bandwidth_group, TntEdge *edge);

#endif  /* TNT_BANDWIDTH_GROUP_H */
