/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-block-db.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_BLOCK_DB_H
#define TNT_BLOCK_DB_H

#include <dirent.h>

#define TNT_BLOCK_DB(object) ((TntBlockDB *) object)

typedef enum
{
  TNT_BLOCK_DB_IDLE,
  TNT_BLOCK_DB_SCANNING,
  TNT_BLOCK_DB_ACTION
}
TntBlockDBState;

struct _TntBlockDB
{
  TntDataDB         data_db;

  TntRetrievalList *retrieval_list;
  GArray           *known_files;
  GList            *block_action_queue;
  TntBlockAction   *current_block_action;

  gint              file_count;
  gint              file_complete_count;
  gint64            file_size;
  gint64            file_complete_size;

  TntBlockDBState   state;

  /* State information */
  guint             callback_source_id;
  DIR              *top_dir_hd;
  DIR              *sub_dir_hd;
  FluxFileIO       *file_io;
  gpointer          block;
};

gboolean     tnt_block_db_init                     (TntBlockDB *block_db, TntStorage *storage);
TntBlockDB  *tnt_block_db_new                      (TntStorage *storage);
void         tnt_block_db_finalize                 (FluxObject *object);

void         tnt_block_db_set_retrieval_list       (TntBlockDB *block_db, TntRetrievalList *retrieval_list);

guint        tnt_block_db_get_file_count           (TntBlockDB *block_db, gboolean include_incomplete);
guint64      tnt_block_db_get_file_size            (TntBlockDB *block_db, gboolean include_incomplete);

#endif  /* TNT_BLOCK_DB_H */
