/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-callback-list.c - List of callbacks.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include "tnt.h"

#define DEBUG(x)

#define assert_types(o) flux_object_check (FLUX_OBJECT (o), TNT_CALLBACK_LIST_TYPE)

void
tnt_callback_list_finalize (FluxObject *object)
{
  TntCallbackList *callback_list = TNT_CALLBACK_LIST (object);
  GList           *l;

  for (l = callback_list->callback_list; l; l = g_list_next (l))
  {
    g_free (l->data);
  }

  g_list_free (callback_list->callback_list);
}

void
tnt_callback_list_init (TntCallbackList *callback_list)
{
  flux_object_init              (FLUX_OBJECT (callback_list));
  flux_object_set_type          (FLUX_OBJECT (callback_list), TNT_CALLBACK_LIST_TYPE);
  flux_object_set_finalize_func (FLUX_OBJECT (callback_list), tnt_callback_list_finalize);

  callback_list->callback_list = NULL;
}

TntCallbackList *
tnt_callback_list_new (void)
{
  TntCallbackList *callback_list;

  callback_list = g_new0 (TntCallbackList, 1);
  tnt_callback_list_init (callback_list);

  return callback_list;
}

void
tnt_callback_list_add_callback (TntCallbackList *callback_list, gpointer func, gpointer data)
{
  TntCallback *callback;

  callback = g_new (TntCallback, 1);
  callback->func = func;
  callback->data = data;

  callback_list->callback_list = g_list_append (callback_list->callback_list, callback);
}

void
tnt_callback_list_remove_callback (TntCallbackList *callback_list, gpointer func, gpointer data)
{
  GList *l;

  for (l = callback_list->callback_list; l; l = g_list_next (l))
  {
    TntCallback *callback = l->data;

    if (callback->func == func && callback->data == data)
    {
      g_free (callback);
      callback_list->callback_list = g_list_delete_link (callback_list->callback_list, l);
      break;
    }
  }
}

#if 0

static gboolean
cb_1 (gint a, gint b, gpointer data)
{
  g_print ("1: %d, %d\n", a, b);
  return TRUE;
}

static gboolean
cb_2 (gint a, gint b, gpointer data)
{
  g_print ("2: %d, %d\n", a, b);
  return FALSE;
}

static gboolean
cb_3 (gint a, gint b, gpointer data)
{
  g_print ("3: %d, %d\n", a, b);
  return TRUE;
}

gint
main (gint argc, gchar *argv [])
{
  TntCallbackList *cb_list;
  gboolean         result;

  cb_list = tnt_callback_list_new ();
  tnt_callback_list_add_callback (cb_list, cb_1, NULL);
  tnt_callback_list_add_callback (cb_list, cb_2, NULL);
  tnt_callback_list_add_callback (cb_list, cb_3, NULL);

  tnt_callback_list_run_while (cb_list, tnt_test_true, 1, 5);
  g_print ("---\n");
 
  tnt_callback_list_run_while_with_result (cb_list, gboolean, result, TRUE, tnt_test_true, 1, 5);
  g_print ("Result: %d\n", result);

  tnt_callback_list_remove_callback (cb_list, cb_1, NULL);
  tnt_callback_list_remove_callback (cb_list, cb_2, NULL);
  tnt_callback_list_remove_callback (cb_list, cb_3, NULL);

  tnt_callback_list_run_while_with_result (cb_list, gboolean, result, TRUE, tnt_test_true, 1, 5);
  g_print ("Default result: %d\n", result);
}

#endif
