/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-class.c - Register/unregister Tentacles classes.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "tnt.h"

typedef struct
{
  gchar *name;
  gchar *parent_name;
}
TntClassInfo;

static TntClassInfo class_info [] =
{
  { "TntContextFactory",     "FluxObject"       },
  { "TntCallbackList",       "FluxObject"       },
  { "TntFileBitmap",         "FluxObject"       },
  { "TntStorage",            "FluxObject"       },
  { "TntConfig",             "FluxObject"       },
  { "TntStringIndex",        "FluxObject"       },
  { "TntStringIndexCursor",  "FluxCursor"       },
  { "TntBlockAction",        "FluxObject"       },
  { "TntQuery",              "FluxObject"       },
  { "TmtLocalQuery",         "TntQuery"         },
  { "TntRemoteQuery",        "TntQuery"         },
  { "TntRetrieval",          "FluxObject"       },
  { "TntRetrievalList",      "FluxObject"       },
  { "TntMetaDB",             "FluxObject"       },
  { "TntDataDB",             "FluxObject"       },
  { "TntFileDB",             "TntDataDB"        },
  { "TntBlockDB",            "TntDataDB"        },
  { "TntFileAssemblerDB",    "FluxObject"       },
  { "TntTcpListener",        "FluxTcpListener"  },
  { "TntTcpIO",              "FluxTcpIO"        },
  { "TntBandwidthGroup",     "FluxObject"       },
  { "TntBandwidthGroupList", "FluxObject"       },
  { "TntMessage",            "FluxObject"       },
  { "TntRoutedMessage",      "TntMessage"       },
  { "TntMessageFilter",      "FluxObject"       },
  { "TntNetDB",              "FluxObject"       },
  { "TntEdge",               "FluxObject"       },
  { "TntTimeWarp",           "FluxObject"       },
  { "TntDisseminator",       "FluxObject"       },
  { "TntCore",               "FluxObject"       },
  { "TntNetworkCore",        "TntCore"          },
  { "TntFileCore",           "TntCore"          }
};

guint tnt_class_types [TNT_CLASSES_NUM];

void
tnt_register_classes (void)
{
  gint i;

  for (i = 0; i < TNT_CLASSES_NUM; i++)
  {
    guint parent_type;

    parent_type = flux_get_class_type (class_info [i].parent_name);
    tnt_class_types [i] = flux_register_class (class_info [i].name, parent_type);
  }
}

void
tnt_unregister_classes (void)
{
  gint i;

  for (i = 0; i < TNT_CLASSES_NUM; i++)
  {
    flux_unregister_class_by_type (tnt_class_types [i]);
  }
}
