/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-disseminator.h - Maintains a queue of blocks to send.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_DISSEMINATOR_H
#define TNT_DISSEMINATOR_H

#define TNT_DISSEMINATOR(object) ((TntDisseminator *) object)

struct _TntDisseminator
{
  FluxObject         object;

  GList             *queue, *last;
  gint               queue_len;

  GList             *blocked_nodes;
};

void             tnt_disseminator_init             (TntDisseminator *disseminator);
TntDisseminator *tnt_disseminator_new              (void);
void             tnt_disseminator_finalize         (FluxObject *disseminator);

gint             tnt_disseminator_get_queue_len    (TntDisseminator *disseminator);

gboolean         tnt_disseminator_enqueue          (TntDisseminator *disseminator, TntNodeID node_id,
                                                    const TntFileID *file_id, gint block_id);
gboolean         tnt_disseminator_dequeue          (TntDisseminator *disseminator, TntNodeID *node_id,
                                                    TntFileID *file_id, gint *block_id);
gboolean         tnt_disseminator_peek             (TntDisseminator *disseminator, TntNodeID *node_id,
                                                    TntFileID *file_id, gint *block_id);

gboolean         tnt_disseminator_dequeue_nth      (TntDisseminator *disseminator, gint n,
                                                    TntNodeID *node_id, TntFileID *file_id, gint *block_id);
gboolean         tnt_disseminator_peek_nth         (TntDisseminator *disseminator, gint n,
                                                    TntNodeID *node_id, TntFileID *file_id, gint *block_id);

gboolean         tnt_disseminator_find_node        (TntDisseminator *disseminator, TntNodeID node_id,
                                                    TntFileID *file_id, gint *block_id);

void             tnt_disseminator_block_node       (TntDisseminator *disseminator, TntNodeID node_id);
void             tnt_disseminator_unblock_node     (TntDisseminator *disseminator, TntNodeID node_id);

#endif  /* TNT_DISSEMINATOR_H */
