/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-file-assembler.h - Assembles files from blocks to a serial file.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_FILE_ASSEMBLER_H
#define TNT_FILE_ASSEMBLER_H

#define TNT_FILE_ASSEMBLER(o) ((TntFileAssembler *) (o))

struct _TntFileAssembler
{
  FluxObject  object;

  GHashTable *assembly_table;
  gpointer    current_assembly;

  TntDataDB  *source_db;
  TntFileDB  *dest_db;
};

typedef enum
{
  TNT_FILE_ASSEMBLY_UNKNOWN,
  TNT_FILE_ASSEMBLY_WAITING,
  TNT_FILE_ASSEMBLY_RUNNING,
  TNT_FILE_ASSEMBLY_COMPLETE,

  TNT_FILE_ASSEMBLY_ERROR_INCOMPLETE,    /* Incomplete in source DB */
  TNT_FILE_ASSEMBLY_ERROR_NO_SPACE,      /* No space on target */
  TNT_FILE_ASSEMBLY_ERROR_FILESYS,       /* Permission/path problem on target */
  TNT_FILE_ASSEMBLY_ERROR_BAD_CHECKSUM,  /* Complete, but file ID mismatch */
  TNT_FILE_ASSEMBLY_ERROR_OTHER_SOURCE,  /* Error reading from source */
  TNT_FILE_ASSEMBLY_ERROR_OTHER_TARGET,  /* Error writing to target */
  TNT_FILE_ASSEMBLY_ERROR_OTHER,

  TNT_FILE_ASSEMBLY_FIRST_ERROR = TNT_FILE_ASSEMBLY_ERROR_INCOMPLETE,
  TNT_FILE_ASSEMBLY_LAST_ERROR  = TNT_FILE_ASSEMBLY_ERROR_OTHER
}
TntFileAssemblyStatus;

TntFileAssembler     *tnt_file_assembler_new                 (TntDataDB *source_db, TntFileDB *dest_db);
gboolean              tnt_file_assembler_init                (TntFileAssembler *assembler,
                                                              TntDataDB *source_db, TntFileDB *dest_db);
void                  tnt_file_assembler_finalize            (FluxObject *object);

gboolean              tnt_file_assembler_add_assembly        (TntFileAssembler *assembler,
                                                              const TntFileID *file_id,
                                                              const gchar *file_path,
                                                              const gchar *file_name,
                                                              TntAssemblyFunc *func, gpointer data);
gboolean              tnt_file_assembler_remove_assembly     (TntFileAssembler *assembler,
                                                              const TntFileID *file_id);

TntFileAssemblyStatus tnt_file_assembler_get_assembly_status (TntFileAssembler *assembler,
                                                              const TntFileID *file_id);

#endif
