/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-file-id.c - File ID.
 *
 * Copyright (C) 2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <string.h>
#include "tnt.h"

FluxMarshalledStructEntry tnt_file_id_m_entries [] =
{
  FLUX_MARSHAL_RAW_FIXED      ("hash", TntFileID, hash, TRUE, NULL),
  FLUX_MARSHAL_UINT64         ("len",  TntFileID, len,  TRUE, 0),
  FLUX_MARSHAL_END            ()
};

guint
tnt_file_id_hash (const TntFileID *file_id)
{
  guint val = 0;
  gint  i;

  for (i = 0; i < sizeof (file_id->hash); i++)
  {
    guint t;

    val ^= ((const guchar *) file_id->hash) [i];
    t = val & (0xf0 << (sizeof (guint) * 8 - 8));
    val <<= 4;
    val |= t >> (sizeof (guint) * 8 - 4);
  }

  for (i = 0; i < sizeof (file_id->len); i++)
  {
    guint t;

    val ^= ((const guchar *) &file_id->len) [i];
    t = val & (0xf0 << (sizeof (guint) * 8 - 8));
    val <<= 4;
    val |= t >> (sizeof (guint) * 8 - 4);
  }

  return val;
}

gboolean
tnt_file_id_equal (const TntFileID *file_id_1, const TntFileID *file_id_2)
{
  if (file_id_1->len != file_id_2->len)
    return FALSE;

  if (memcmp (file_id_1->hash, file_id_2->hash, TNT_FILE_ID_LEN))
    return FALSE;

  return TRUE;
}
