/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-file-meta.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_FILE_META_H
#define TNT_FILE_META_H

#define TNT_FILE_META(s) ((TntFileMeta *) (s))

#define TNT_LANG_UNSPECIFIED     0
#define TNT_LANG_OTHER           0xff

#define TNT_CATEGORY_UNSPECIFIED 0
#define TNT_CATEGORY_OTHER       0xff

typedef enum
{
  TNT_FILE_META_USER_INPUT        = (1 << 0),
  TNT_FILE_META_LOCAL_USER_INPUT  = (1 << 1),
  TNT_FILE_META_LOCAL_FILE        = (1 << 2)
}
TntFileMetaFlag;

typedef struct
{
  gchar *title;
  gchar *collection;
  gchar *performer;
  gchar *author;
  gchar *keywords;
  gchar *width;
  gchar *height;
  gchar *duration;
  gchar *seq;
  gchar *year;
  gchar *fragment;
  gchar *category;
  gchar *lang;
}
TntMediaLabels;

struct _TntFileMeta
{
  TntTimeStamp timestamp;

  guchar  *name;
  guchar  *title;
  guchar  *collection;
  guchar  *performer;
  guchar  *author;
  guchar  *keywords;
  guchar  *seq;
  guint32  duration;      /* seconds */
  guint16  width;         /* pixels */
  guint16  height;        /* pixels */
  guint16  fragment;      /* fragment # */
  guint16  max_fragment;  /* total fragments in same group */
  guint16  quality;       /* bitrate in kbps */
  guint16  format;
  gint16   year;
  guint8   month;
  guint8   day;
  guint8   lang [2];
  guint8   cat  [2];
  guint8   media;
  guint8   flags;
};

extern TntMediaLabels            media_labels [];
extern FluxMarshalledStructEntry tnt_file_meta_m_entries [];

#define        tnt_media_label(type, name)        _tnt_get_media_label (type, #name)
const gchar   *_tnt_get_media_label               (gint type, const gchar *name);
const gchar   *tnt_media_name                     (TntMediaType media);

TntFileMeta   *tnt_file_meta_new                  (void);
TntFileMeta   *tnt_file_meta_dup                  (TntFileMeta *meta);
void           tnt_file_meta_free                 (TntFileMeta *meta);
void           tnt_file_meta_reclaim_memory       (void);

TntFileMeta   *tnt_file_meta_merge                (TntFileMeta *a, TntFileMeta *b);
gboolean       tnt_file_meta_strip_foreign        (TntFileMeta *file_meta);

gboolean       tnt_file_meta_is_local_user_input  (TntFileMeta *meta);
void           tnt_file_meta_set_local_user_input (TntFileMeta *meta, gboolean value);
gboolean       tnt_file_meta_is_user_input        (TntFileMeta *meta);
void           tnt_file_meta_set_user_input       (TntFileMeta *meta, gboolean value);

gboolean       tnt_file_meta_is_local_file        (TntFileMeta *meta);
void           tnt_file_meta_set_local_file       (TntFileMeta *meta, gboolean is_local);

TntTimeStamp   tnt_file_meta_get_timestamp        (TntFileMeta *meta);
void           tnt_file_meta_set_timestamp        (TntFileMeta *meta, TntTimeStamp timestamp);

TntFormatType  tnt_file_meta_get_format           (TntFileMeta *meta);
void           tnt_file_meta_set_format           (TntFileMeta *meta, TntFormatType format);
guint8         tnt_file_meta_get_media            (TntFileMeta *meta);
void           tnt_file_meta_set_media            (TntFileMeta *meta, guint8 media);
gchar         *tnt_file_meta_get_name             (TntFileMeta *meta);
void           tnt_file_meta_set_name             (TntFileMeta *meta, const gchar *name);
gchar         *tnt_file_meta_get_title            (TntFileMeta *meta);
void           tnt_file_meta_set_title            (TntFileMeta *meta, const gchar *title);
gchar         *tnt_file_meta_get_collection       (TntFileMeta *meta);
void           tnt_file_meta_set_collection       (TntFileMeta *meta, const gchar *collection);
gchar         *tnt_file_meta_get_performer        (TntFileMeta *meta);
void           tnt_file_meta_set_performer        (TntFileMeta *meta, const gchar *performer);
gchar         *tnt_file_meta_get_author           (TntFileMeta *meta);
void           tnt_file_meta_set_author           (TntFileMeta *meta, const gchar *author);
gchar         *tnt_file_meta_get_keywords         (TntFileMeta *meta);
void           tnt_file_meta_set_keywords         (TntFileMeta *meta, const gchar *keywords);
guint16        tnt_file_meta_get_width            (TntFileMeta *meta);
void           tnt_file_meta_set_width            (TntFileMeta *meta, guint16 width);
guint16        tnt_file_meta_get_height           (TntFileMeta *meta);
void           tnt_file_meta_set_height           (TntFileMeta *meta, guint16 height);
guint32        tnt_file_meta_get_duration         (TntFileMeta *meta);
void           tnt_file_meta_set_duration         (TntFileMeta *meta, guint32 duration);
gchar         *tnt_file_meta_get_seq              (TntFileMeta *meta);
void           tnt_file_meta_set_seq              (TntFileMeta *meta, const gchar *seq);
guint16        tnt_file_meta_get_quality          (TntFileMeta *meta);
void           tnt_file_meta_set_quality          (TntFileMeta *meta, guint16 quality);
guint16        tnt_file_meta_get_fragment         (TntFileMeta *meta);
void           tnt_file_meta_set_fragment         (TntFileMeta *meta, guint16 fragment);
guint16        tnt_file_meta_get_max_fragment     (TntFileMeta *meta);
void           tnt_file_meta_set_max_fragment     (TntFileMeta *meta, guint16 max_fragment);
gint16         tnt_file_meta_get_year             (TntFileMeta *meta);
void           tnt_file_meta_set_year             (TntFileMeta *meta, gint16 year);
guint8         tnt_file_meta_get_month            (TntFileMeta *meta);
void           tnt_file_meta_set_month            (TntFileMeta *meta, guint8 month);
guint8         tnt_file_meta_get_day              (TntFileMeta *meta);
void           tnt_file_meta_set_day              (TntFileMeta *meta, guint8 day);
guint8         tnt_file_meta_get_lang             (TntFileMeta *meta, gint n);
void           tnt_file_meta_set_lang             (TntFileMeta *meta, gint n, guint8 lang);
guint8         tnt_file_meta_get_category         (TntFileMeta *meta, gint n);
void           tnt_file_meta_set_category         (TntFileMeta *meta, gint n, guint8 category);

gchar         *tnt_file_meta_get_best_title       (TntFileMeta *meta);
gchar         *tnt_file_meta_get_best_description (TntFileMeta *meta);

gboolean       tnt_file_meta_matches_string       (TntFileMeta *meta, TntMatchField field,
                                                   const gchar *string);

#endif
