/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-local-query.h - Describes a metadata query for local presentation.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_LOCAL_QUERY_H
#define TNT_LOCAL_QUERY_H

#define TNT_LOCAL_QUERY(object) ((TntLocalQuery *) object)

struct _TntLocalQuery
{
  TntQuery           query;

  GList             *match_pre_add_funcs;
  GList             *match_add_funcs;
  GList             *match_update_funcs;
  GList             *match_remove_funcs;

  TntMatch          *root_match;

  guint              grace_timeout;
  gboolean           have_grace_timeout;

  GHashTable        *file_id_hash;
  GHashTable        *collection_hash;
  GHashTable        *keyword_hash;
};

void           tnt_local_query_init                     (TntLocalQuery *query);
TntLocalQuery *tnt_local_query_new                      (void);
void           tnt_local_query_finalize                 (FluxObject *query);

void           tnt_local_query_add_match_pre_add_func   (TntLocalQuery *queyr,
                                                         TntMatchFunc *func, gpointer data);
void           tnt_local_query_add_match_add_func       (TntLocalQuery *query,
                                                         TntMatchFunc *func, gpointer data);
void           tnt_local_query_add_match_update_func    (TntLocalQuery *query,
                                                         TntMatchFunc *func, gpointer data);
void           tnt_local_query_add_match_remove_func    (TntLocalQuery *query,
                                                         TntMatchFunc *func, gpointer data);

void           tnt_local_query_remove_match_func        (TntLocalQuery *query,
                                                         TntMatchFunc *func, gpointer data);

TntMatch      *tnt_local_query_get_root_match           (TntLocalQuery *query);

TntMatch      *tnt_local_query_get_match_n              (TntLocalQuery *query, gint match_n);
TntMatch      *tnt_local_query_find_match_by_file_id    (TntLocalQuery *query, TntFileID *file_id);
void           tnt_local_query_match_updated            (TntLocalQuery *query, TntMatch *match);

void           tnt_local_query_for_all_matches          (TntLocalQuery *query,
                                                         TntMatchFunc *func, gpointer data);

#endif  /* TNT_LOCAL_QUERY_H */
