/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-meta-db.h - On-disk indexed file metadata.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_META_DB_H
#define TNT_META_DB_H

#define TNT_META_DB(object) ((TntMetaDB *) object)

struct _TntMetaDB
{
  FluxObject            object;

  GMutex               *mutex;
  TntAsyncQueue        *queue;
  GThread              *worker_thread;

  TntStorage           *storage;
  FluxStorage          *id_to_file_meta_storage;
  FluxStorage          *string_storage;
  FluxPatTree          *id_to_file_meta_tree;
  TntStringIndex       *string_to_id_index;

  GPtrArray            *running_queries;
  gint                  running_queries_num;

  guint                 known_count;
  guint64               known_size;
};

gboolean          tnt_meta_db_init              (TntMetaDB *meta_db, TntStorage *storage);
TntMetaDB        *tnt_meta_db_new               (TntStorage *storage);
void              tnt_meta_db_finalize          (FluxObject *meta_db);

TntStorage       *tnt_meta_db_get_storage       (TntMetaDB *meta_db);

void              tnt_meta_db_get_file_meta     (TntMetaDB *meta_db, const TntFileID *file_id,
                                                 TntFileMetaFunc *func, gpointer data);
void              tnt_meta_db_set_file_meta     (TntMetaDB *meta_db, const TntFileID *file_id,
                                                 TntFileMeta *file_meta);
void              tnt_meta_db_merge_file_meta   (TntMetaDB *meta_db, const TntFileID *file_id,
                                                 TntFileMeta *file_meta, gboolean store,
                                                 TntFileMetaFunc *func, gpointer data);
void              tnt_meta_db_extract_file_meta (TntMetaDB *meta_db, const TntFileID *file_id,
                                                 const gchar *file_name, const gchar *file_path,
                                                 const gchar *relative_path,
                                                 TntFileMetaFunc *func, gpointer data);

void              tnt_meta_db_query             (TntMetaDB *meta_db, TntQuery *query, gboolean before,
                                                 TntQueryDoneFunc *func, gpointer data);
void              tnt_meta_db_cancel_query      (TntMetaDB *meta_db, TntQuery *query);
gboolean          tnt_meta_db_query_get_paused  (TntMetaDB *meta_db, TntQuery *query);
void              tnt_meta_db_query_set_paused  (TntMetaDB *meta_db, TntQuery *query, gboolean paused);

void              tnt_meta_db_cancel_callback   (TntMetaDB *meta_db, gpointer func, gpointer data);

void              tnt_meta_db_get_local_file_id (TntMetaDB *meta_db, const gchar *name, guint64 len,
                                                 TntTimeStamp timestamp, TntFileIDFunc *func,
                                                 gpointer data);

guint             tnt_meta_db_get_file_count    (TntMetaDB *meta_db);
guint64           tnt_meta_db_get_file_size     (TntMetaDB *meta_db);

gint              tnt_meta_db_get_action_queue_len (TntMetaDB *meta_db);

#endif  /* TNT_META_DB_H */
