/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-network-messages.c - Defines messages used by TntNetworkCore.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include "tnt.h"
#include "tnt-network-messages.h"

typedef struct
{
  TntMessageType  type;
  gchar          *name;
}
MessageTypeName;

/* --- Secondary datatypes --- */

static FluxMarshalledStructEntry node_address_4_m_entries [] =
{
  FLUX_MARSHAL_UINT32         ("ip",   TntNodeAddress4, ip,   TRUE, 0),
  FLUX_MARSHAL_UINT16         ("port", TntNodeAddress4, port, TRUE, 0),
  FLUX_MARSHAL_END            ()
};

static FluxMarshalledStructEntry node_meta_m_entries [] =
{
  FLUX_MARSHAL_UINT32         ("last-seen",         TntNodeMeta, last_seen,         TRUE, 0),
  FLUX_MARSHAL_STRUCT_FIXED   ("primary-address",   TntNodeMeta, primary_address,   TRUE, "node-address-4"),
  FLUX_MARSHAL_STRUCT_FIXED   ("secondary-address", TntNodeMeta, secondary_address, TRUE, "node-address-4"),
  FLUX_MARSHAL_UINT32         ("primary-rating",    TntNodeMeta, primary_rating,    TRUE, 0),
  FLUX_MARSHAL_UINT32         ("secondary-rating",  TntNodeMeta, secondary_rating,  TRUE, 0),
  FLUX_MARSHAL_UINT16         ("flags",             TntNodeMeta, flags,             TRUE, 0),
  FLUX_MARSHAL_END            ()
};

/* --- Message datatypes --- */

static FluxMarshalledStructEntry connect_request_m_entries [] =
{
  FLUX_MARSHAL_UINT64         ("node-id", TntConnectRequestM, id,      TRUE, 0),
  FLUX_MARSHAL_STRUCT_FIXED   ("address", TntConnectRequestM, address, TRUE, "node-address-4"),
  FLUX_MARSHAL_END            ()
};

static FluxMarshalledStructEntry peer_info_m_entries [] =
{
  FLUX_MARSHAL_UINT64         ("id",             TntPeerInfoM, id,             TRUE, 0),
  FLUX_MARSHAL_UINT64         ("fingerprint",    TntPeerInfoM, fingerprint,    TRUE, 0),
  FLUX_MARSHAL_UINT16         ("listening-port", TntPeerInfoM, listening_port, TRUE, 0),
  FLUX_MARSHAL_END            ()
};

static FluxMarshalledStructEntry peer_reverse_info_m_entries [] =
{
  FLUX_MARSHAL_UINT32         ("ip", TntPeerReverseInfoM, ip4, TRUE, 0),
  FLUX_MARSHAL_END            ()
};

static FluxMarshalledStructEntry node_info_m_entries [] =  /* V03 */
{
  FLUX_MARSHAL_UINT64         ("id",   TntNodeInfoM, id,   TRUE, 0),
  FLUX_MARSHAL_STRING_FIXED   ("name", TntNodeInfoM, name, TRUE, NULL),
  FLUX_MARSHAL_END            ()
};

static FluxMarshalledStructEntry contact_info_m_entries [] =
{
  FLUX_MARSHAL_UINT64         ("id",      TntContactInfoM, id,      TRUE, 0),
  FLUX_MARSHAL_STRUCT_FIXED   ("address", TntContactInfoM, address, TRUE, "node-address-4"),
  FLUX_MARSHAL_END            ()
};

static MessageTypeName type_to_name [] =
{
  { TNT_MESSAGE_CONNECT_REQUEST,   "connect-request"   },
  { TNT_MESSAGE_PEER_INFO,         "peer-info"         },
  { TNT_MESSAGE_PEER_REVERSE_INFO, "peer-reverse-info" },
  { TNT_MESSAGE_NODE_INFO,         "node-info"         },
  { TNT_MESSAGE_CONTACT_INFO,      "contact-info"      },
  { TNT_MESSAGE_UNKNOWN,           NULL                }
};

#define F FLUX_MARSHALLER_CLEAR

static FluxMarshalledStruct marshal_info [] =
{
  { "node-address-4",    sizeof (TntNodeAddress4),     F, node_address_4_m_entries    },
  { "node-meta",         sizeof (TntNodeMeta),         F, node_meta_m_entries         },

  { "connect-request",   sizeof (TntConnectRequestM),  F, connect_request_m_entries   },
  { "peer-info",         sizeof (TntPeerInfoM),        F, peer_info_m_entries         },
  { "peer-reverse-info", sizeof (TntPeerReverseInfoM), F, peer_reverse_info_m_entries },
  { "node-info",         sizeof (TntNodeInfoM),        F, node_info_m_entries         },
  { "contact-info",      sizeof (TntContactInfoM),     F, contact_info_m_entries      },
  { NULL }
};

void
tnt_network_messages_register (void)
{
  static gboolean registered = FALSE;
  gint            i;

  if (!registered)
  {
    tnt_message_register_marshal_info (marshal_info);

    for (i = 0; type_to_name [i].name; i++)
      tnt_message_register_type (type_to_name [i].type, type_to_name [i].name);

    registered = TRUE;
  }
}
