/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-query.h - Abstract class for queries.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_QUERY_H
#define TNT_QUERY_H

#define TNT_QUERY(object) ((TntQuery *) object)

struct _TntQuery
{
  FluxObject         object;

  guint16            id;
  guint16            match_local   : 1;
  guint16            match_partial : 1;

  gchar             *string;
  TntMatchField      match_field;
  TntMediaType       media;
  gint               category;

  void               (*filter_func)        (TntQuery *query);
  gint               (*get_n_matches_func) (TntQuery *query);
  gboolean           (*try_match_func)     (TntQuery *query, const TntFileID *file_id,
                                            TntFileMeta *file_meta, TntNodeID node_id,
                                            gint remote_blocks);
};

void           tnt_query_init                     (TntQuery *query);
void           tnt_query_finalize                 (FluxObject *query);

void           tnt_query_set_id                   (TntQuery *query, guint16 id);
guint16        tnt_query_get_id                   (TntQuery *query);

gint           tnt_query_compare                  (TntQuery *query_1, TntQuery *query_2);

const gchar   *tnt_query_get_string               (TntQuery *query);
void           tnt_query_set_string               (TntQuery *query, const gchar *string);

TntMatchField  tnt_query_get_match_field          (TntQuery *query);
void           tnt_query_set_match_field          (TntQuery *query, TntMatchField field);

TntMediaType   tnt_query_get_media                (TntQuery *query);
void           tnt_query_set_media                (TntQuery *query, TntMediaType media);

gint           tnt_query_get_category             (TntQuery *query);
void           tnt_query_set_category             (TntQuery *query, gint category);

gboolean       tnt_query_get_match_local          (TntQuery *query);
void           tnt_query_set_match_local          (TntQuery *query, gboolean match_local);

gboolean       tnt_query_get_match_partial        (TntQuery *query);
void           tnt_query_set_match_partial        (TntQuery *query, gboolean match_partial);

gint           tnt_query_get_n_matches            (TntQuery *query);

gboolean       tnt_query_try_match                (TntQuery *query, const TntFileID *file_id,
                                                   TntFileMeta *file_meta, TntNodeID node_id,
                                                   gint remote_blocks);

#endif  /* TNT_QUERY_H */
