/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-retrieval-list.h - Holds a list of file retrievals.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_RETRIEVAL_LIST_H
#define TNT_RETRIEVAL_LIST_H

#define TNT_RETRIEVAL_LIST(object) ((TntRetrievalList *) object)

struct _TntRetrievalList
{
  FluxObject         object;

  TntStorage        *storage;
  TntStorage        *incoming_storage;

  gint               save_timeout_id;
  guint              poll_timeout_id;

  GSList            *retrieval_add_funcs;
  GSList            *retrieval_update_funcs;
  GSList            *retrieval_remove_funcs;
  GSList            *retrieval_completed_funcs;

  GPtrArray         *retrievals;

  GSList            *callback_pos_list;
};

gboolean          tnt_retrieval_list_init             (TntRetrievalList *retrieval_list, TntStorage *storage);
TntRetrievalList *tnt_retrieval_list_new              (TntStorage *storage);
void              tnt_retrieval_list_finalize         (FluxObject *retrieval_list);

void              tnt_retrieval_list_set_incoming_storage (TntRetrievalList *retrieval_list, TntStorage *storage);

void              tnt_retrieval_list_load             (TntRetrievalList *retrieval_list, TntBlockDB *block_db);

void              tnt_retrieval_list_add_retrieval_add_func    (TntRetrievalList *retrieval_list,
                                                               TntRetrievalListFunc *func, gpointer data);
void              tnt_retrieval_list_add_retrieval_update_func (TntRetrievalList *retrieval_list,
                                                               TntRetrievalListFunc *func, gpointer data);
void              tnt_retrieval_list_add_retrieval_remove_func (TntRetrievalList *retrieval_list,
                                                               TntRetrievalListFunc *func, gpointer data);
void              tnt_retrieval_list_add_retrieval_completed_func (TntRetrievalList *retrieval_list,
                                                                  TntRetrievalListFunc *func, gpointer data);

void              tnt_retrieval_list_remove_retrieval_func     (TntRetrievalList *retrieval_list,
                                                                TntRetrievalListFunc *func, gpointer data);

gint              tnt_retrieval_list_get_n_retrievals (TntRetrievalList *retrieval_list);

/* Retrievals that are active but incomplete */
gint              tnt_retrieval_list_get_n_incomplete_retrievals (TntRetrievalList *retrieval_list);

gint              tnt_retrieval_list_find_retrieval   (TntRetrievalList *retrieval_list, const TntFileID *file_id);
TntRetrieval     *tnt_retrieval_list_get_retrieval_n  (TntRetrievalList *retrieval_list, gint retrieval_n);
gint              tnt_retrieval_list_add_retrieval    (TntRetrievalList *retrieval_list, TntRetrieval *retrieval);
void              tnt_retrieval_list_remove_retrieval (TntRetrievalList *retrieval_list, TntRetrieval *retrieval);
gboolean          tnt_retrieval_list_move_retrieval   (TntRetrievalList *retrieval_list, gint old_n, gint new_n);

void              tnt_retrieval_list_updated          (TntRetrievalList *retrieval_list, gint retrieval_n);

gboolean          tnt_retrieval_list_ask_block        (TntRetrievalList *retrieval_list, TntFileID *file_id,
                                                       TntNodeID *node_id, gint *block_id);

void              tnt_retrieval_list_on_queue         (TntRetrievalList *retrieval_list, const TntFileID *file_id,
                                                       TntNodeID node_id, gint block_id,
                                                       TntTimeStamp remote_timestamp);
void              tnt_retrieval_list_off_queue        (TntRetrievalList *retrieval_list, TntNodeID node_id,
                                                       gint block_id, TntTimeStamp remote_timestamp);
void              tnt_retrieval_list_got_block        (TntRetrievalList *retrieval_list, const TntFileID *file_id,
                                                       gint block_id);

#endif  /* TNT_RETRIEVAL_LIST_H */
