/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-routed-message.c - Message with routing headers.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "tnt.h"
#include <string.h>

#define assert_types(o) flux_object_check (FLUX_OBJECT (o), TNT_ROUTED_MESSAGE_TYPE)

void
tnt_routed_message_finalize (FluxObject *rmessage)
{
  assert_types (rmessage);

  if (TNT_MESSAGE (rmessage)->destroy_func)
  {
    TNT_MESSAGE (rmessage)->destroy_func (rmessage, TNT_MESSAGE (rmessage)->destroy_func_data);
    TNT_MESSAGE (rmessage)->destroy_func = NULL;
  }

  tnt_message_finalize (rmessage);
}

void
tnt_routed_message_init (TntRoutedMessage *rmessage, TntMessageType type,
                         TntNodeID source_id, TntNodeID dest_id)
{
  tnt_message_init              (TNT_MESSAGE  (rmessage), type);
  flux_object_set_type          (FLUX_OBJECT (rmessage), TNT_ROUTED_MESSAGE_TYPE);
  flux_object_set_finalize_func (FLUX_OBJECT (rmessage), tnt_routed_message_finalize);

  rmessage->source_id = source_id;
  rmessage->dest_id   = dest_id;
}

TntRoutedMessage *
tnt_routed_message_new (TntMessageType type, TntNodeID source_id, TntNodeID dest_id)
{
  TntRoutedMessage *rmessage;

  rmessage = g_new0 (TntRoutedMessage, 1);
  tnt_routed_message_init (rmessage, type, source_id, dest_id);

  return rmessage;
}

gboolean
tnt_routed_message_has_source (TntRoutedMessage *message)
{
  assert_types (message);

  if (message->source_id == TNT_NODE_ID_INVALID)
    return FALSE;

  return TRUE;
}

gboolean
tnt_routed_message_has_destination (TntRoutedMessage *message)
{
  assert_types (message);

  if (message->dest_id == TNT_NODE_ID_INVALID)
    return FALSE;

  return TRUE;
}

TntNodeID
tnt_routed_message_get_source (TntRoutedMessage *message)
{
  assert_types (message);

  return message->source_id;
}

TntNodeID
tnt_routed_message_get_destination (TntRoutedMessage *message)
{
  assert_types (message);

  return message->dest_id;
}
