/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-tcp-listener.c - Listener for TCP connections.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <glib.h>

#ifndef G_PLATFORM_WIN32
# include <unistd.h>
#else
# include <io.h>
#endif

#include "tnt.h"

static void
connect_cb (FluxTcpListener *tl, FluxTcpIO *io, gpointer data)
{
  flux_object_assert (FLUX_OBJECT (tl));
  flux_object_assert (FLUX_OBJECT (io));

  if (TNT_TCP_LISTENER (tl)->connection_func)
  {
    tnt_tcp_io_init_self (TNT_TCP_IO (io));

    TNT_TCP_LISTENER (tl)->connection_func (TNT_TCP_LISTENER (tl), TNT_TCP_IO (io),
                                            TNT_TCP_LISTENER (tl)->connection_func_data);
  }
  else
  {
    flux_object_unref (FLUX_OBJECT (io));
  }
}

gboolean
tnt_tcp_listener_init (TntTcpListener *tl, gint port, FluxCipher *cipher)
{
  if (!flux_tcp_listener_init (FLUX_TCP_LISTENER (tl), port, cipher))
    return FALSE;

  flux_tcp_listener_set_compression (FLUX_TCP_LISTENER (tl), 4, 4);

  tl->tcp_listener.spawned_object_size = sizeof (TntTcpIO);

  flux_object_set_type          (FLUX_OBJECT (tl), TNT_TCP_LISTENER_TYPE);
  flux_object_set_finalize_func (FLUX_OBJECT (tl), tnt_tcp_listener_finalize);

  flux_tcp_listener_set_connection_func (FLUX_TCP_LISTENER (tl), connect_cb, NULL);
  return TRUE;
}

TntTcpListener *
tnt_tcp_listener_new (gint port)
{
  TntTcpListener *tl;

  tl = g_new0 (TntTcpListener, 1);

  if (!tnt_tcp_listener_init (tl, port, NULL))
  {
    g_free (tl);
    tl = NULL;
  }

  return tl;
}

TntTcpListener *
tnt_tcp_listener_new_with_cipher (gint port, FluxCipher *cipher)
{
  TntTcpListener *tl;

  tl = tnt_tcp_listener_new (port);
  if (tl)
    flux_tcp_listener_set_cipher (FLUX_TCP_LISTENER (tl), cipher);

  return tl;
}

void
tnt_tcp_listener_finalize (FluxObject *tl)
{
  flux_tcp_listener_finalize (tl);
}

void
tnt_tcp_listener_set_connection_func (TntTcpListener *tl, TntTcpConnectionFunc *func, gpointer data)
{
  flux_object_check (FLUX_OBJECT (tl), TNT_TCP_LISTENER_TYPE);

  tl->connection_func      = func;
  tl->connection_func_data = data;
}
