/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* tnt-time-warp.h - Time-delayed callback source for TntMessage.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef TNT_TIME_WARP_H
#define TNT_TIME_WARP_H

#define TNT_TIME_WARP(object) ((TntTimeWarp *) object)

struct _TntTimeWarp
{
  FluxObject         object;

  TntNetworkCore    *network_core;  /* No formal reference is kept */
  GTree             *message_tree;
  gdouble            min_delay;
  gdouble            max_delay;

  guint64            timeout_ms;
  guint              timeout_id;

  TntMessageHandler *message_handler;
  gpointer           message_handler_data;
};

void             tnt_time_warp_init                     (TntTimeWarp *time_warp,
                                                         TntNetworkCore *network_core);
TntTimeWarp     *tnt_time_warp_new                      (TntNetworkCore *network_core);
void             tnt_time_warp_finalize                 (FluxObject *time_warp);

void             tnt_time_warp_push_message             (TntTimeWarp *time_warp,
                                                         TntMessage *message, TntEdge *source_edge);

void             tnt_time_warp_get_delay                (TntTimeWarp *time_warp,
                                                         gdouble *min_delay, gdouble *max_delay);
void             tnt_time_warp_set_delay                (TntTimeWarp *time_warp,
                                                         gdouble min_delay, gdouble max_delay);

void             tnt_time_warp_set_message_handler      (TntTimeWarp *time_warp,
                                                         TntMessageHandler *message_handler,
                                                         gpointer data);

#endif  /* TNT_TIME_WARP_H */
